//Modify ColorWheel sample
// 2014/8/29
//
// LED sample with RCBController
//
// RCBController (iOS app)
// https://itunes.apple.com/app/rcbcontroller/id689724127?mt=8
//
// by Micono Utilities
// micono@mac.com


/*
The sketch receives a Bluetooth Low Energy 4 connection,
then uses PWM on the RGB shield to approximate the color
selected.

This sketch is supposed to be used with the rfduinoColorWheel
iPhone application.
*/

#include <RFduinoBLE.h>

// pin 2 on the RGB shield is the red led
int led1 = 2;
// pin 3 on the RGB shield is the green led
int led2 = 3;
// pin 4 on the RGB shield is the blue led
int led3 = 4;

void setup() {
  // setup the leds for output
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);  
  pinMode(led3, OUTPUT);

  // this is the data we want to appear in the advertisement
  // (the deviceName length plus the advertisement length must be <= 18 bytes
  RFduinoBLE.advertisementData = "rgb";
  
  // start the BLE stack
  RFduinoBLE.begin();
}

void loop() {
  // switch to lower power mode
  RFduino_ULPDelay(INFINITE);
}

void RFduinoBLE_onConnect() {
  // the default starting color on the iPhone is white
  analogWrite(led1, 255);
  analogWrite(led2, 255);
  analogWrite(led3, 255);
}

void RFduinoBLE_onDisconnect() {
  // turn all leds off on disconnect and stop pwm
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
  digitalWrite(led3, LOW);
}

void RFduinoBLE_onReceive(char *data, int len) {
  // HSV
  // Left Analog stick: Hue & Saturation
  // Right Analog stick (up-down): Value 
  if (len >= 10)
  {
    float x,y;
    x=((float)data[2]-128.0f)/128.0f;
    y=((float)data[3]-128.0f)/128.0f;
    float h=(180.0f+atan2(x,y)*180.0f/PI)/360.0f;
    if(h<0.0f) h=0.0f;
    if(h>1.0f) h=1.0f;
    float s=sqrt(x*x+y*y);
    if(s<0.0f) s=0.0f;
    if(s>1.0f) s=1.0f;
    float v=((float)data[5])/255.0f;
    if(v<0.0f) v=0.0f;
    if(v>1.0f) v=1.0f;
    float r = v;
    float g = v;
    float b = v;
    if (s > 0.0f) {
        h *= 6.0f;
        int i = (int) h;
        float f = h - (float) i;
        switch (i) {
            default:
            case 0:
                g *= 1 - s * (1 - f);
                b *= 1 - s;
                break;
            case 1:
                r *= 1 - s * f;
                b *= 1 - s;
                break;
            case 2:
                r *= 1 - s;
                b *= 1 - s * (1 - f);
                break;
            case 3:
                r *= 1 - s;
                g *= 1 - s * f;
                break;
            case 4:
                r *= 1 - s * (1 - f);
                g *= 1 - s;
                break;
            case 5:
                g *= 1 - s;
                b *= 1 - s * f;
                break;
        }
    }
    
    // get the RGB values
    //uint8_t r = data[0];
    //uint8_t g = data[1];
    //uint8_t b = data[2];

    // set PWM for each led
    analogWrite(led1, (uint8_t)(r*255.0));
    analogWrite(led2, (uint8_t)(g*255.0));
    analogWrite(led3, (uint8_t)(b*255.0));
  }
}
